@@ -76,7 +76,7 @@ commands = [u"cleanup",
 
 def old_fn_deprecation(opt):
     log.Log(_(u"Warning: Option %s is pending deprecation "
-              u"and will be removed in a future release.\n"
+              u"and will be removed in version 0.9.0.\n"
               u"Use of default filenames is strongly suggested.") % opt,
             log.ERROR, force_print=True)
 
@@ -99,6 +99,11 @@ def stdin_deprecation(opt):
             log.ERROR, force_print=True)
 
 
+# log options handled in log.py.  Add noop to make optparse happy
+def noop():
+    pass
+
+
 def expand_fn(filename):
     return os.path.expanduser(os.path.expandvars(filename))
 
@@ -240,7 +245,8 @@ def parse_cmdline_options(arglist):
         sys.exit(0)
 
     def add_rename(o, s, v, p):  # pylint: disable=unused-argument
-        config.rename[os.path.normcase(os.path.normpath(v[0]))] = v[1]
+        key = util.fsencode(os.path.normcase(os.path.normpath(v[0])))
+        config.rename[key] = util.fsencode(v[1])
 
     parser = optparse.OptionParser(option_class=DupOption, usage=usage())
 
@@ -430,6 +436,10 @@ def parse_cmdline_options(arglist):
                       dest=u"", action=u"callback",
                       callback=lambda o, s, v, p: log.add_file(v))
 
+    # log option to add timestamp and level to log entries
+    parser.add_option(u"--log-timestamp", action=u"callback",
+                      callback=lambda o, s, v, p: noop())
+
     # Maximum block size for large files
     parser.add_option(u"--max-blocksize", type=u"int", metavar=_(u"number"))
 
@@ -458,6 +468,10 @@ def parse_cmdline_options(arglist):
     # File owner uid keeps number from tar file. Like same option in GNU tar.
     parser.add_option(u"--numeric-owner", action=u"store_true")
 
+    # Do no restore the uid/gid when finished, useful if you're restoring
+    # data without having root privileges or Unix users support
+    parser.add_option(u"--do-not-restore-ownership", action=u"store_true")
+
     # Whether the old filename format is in effect.
     parser.add_option(u"--old-filenames", action=u"callback",
                       dest=u"old_filenames",
@@ -551,6 +565,10 @@ def parse_cmdline_options(arglist):
     parser.add_option(u"--s3-kms-key-id", action=u"store", dest=u"s3_kms_key_id")
     parser.add_option(u"--s3-kms-grant", action=u"store", dest=u"s3_kms_grant")
 
+    # Options for specifying region and endpoint of s3
+    parser.add_option(u"--s3-region-name", type=u"string", dest=u"s3_region_name", action=u"store")
+    parser.add_option(u"--s3-endpoint-url", type=u"string", dest=u"s3_endpoint_url", action=u"store")
+
     # Option to specify a Swift container storage policy.
     parser.add_option(u"--swift-storage-policy", type=u"string", metavar=_(u"policy"))
 
@@ -957,6 +975,7 @@ def usage():
   hsi://%(user)s[:%(password)s]@%(other_host)s[:%(port)s]/%(some_dir)s
   imap://%(user)s[:%(password)s]@%(other_host)s[:%(port)s]/%(some_dir)s
   mega://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
+  megav2://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   mf://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   onedrive://%(some_dir)s
   pca://%(container_name)s
