@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,39 +19,51 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-"""
+u"""
 Functions for producing signatures and deltas of directories
 
 Note that the main processes of this module have two parts.  In the
 first, the signature or delta is constructed of a ROPath iterator.  In
 the second, the ROPath iterator is put into tar block form.
 """
+from __future__ import division
+
+from future import standard_library
+standard_library.install_aliases()
+from builtins import map
+from builtins import next
+from builtins import str
+from builtins import range
+from builtins import object
+
+import io
+import sys
 
-import cStringIO, types, math
 from duplicity import statistics
 from duplicity import util
-from duplicity import globals
-from duplicity.path import * #@UnusedWildImport
-from duplicity.lazy import * #@UnusedWildImport
+from duplicity import config
+from duplicity.path import *  # pylint: disable=unused-wildcard-import,redefined-builtin
+from duplicity.lazy import *  # pylint: disable=unused-wildcard-import,redefined-builtin
 from duplicity import progress
 
 # A StatsObj will be written to this from DirDelta and DirDelta_WriteSig.
 stats = None
 tracker = None
 
+
 class DiffDirException(Exception):
     pass
 
 
 def DirSig(path_iter):
-    """
+    u"""
     Alias for SigTarBlockIter below
     """
     return SigTarBlockIter(path_iter)
 
 
 def DirFull(path_iter):
-    """
+    u"""
     Return a tarblock full backup of items in path_iter
 
     A full backup is just a diff starting from nothing (it may be less
@@ -59,18 +71,18 @@ def DirFull(path_iter):
     will be easy to split up the tar and make the volumes the same
     sizes).
     """
-    return DirDelta(path_iter, cStringIO.StringIO(""))
+    return DirDelta(path_iter, io.StringIO(u""))
 
 
 def DirFull_WriteSig(path_iter, sig_outfp):
-    """
+    u"""
     Return full backup like above, but also write signature to sig_outfp
     """
-    return DirDelta_WriteSig(path_iter, cStringIO.StringIO(""), sig_outfp)
+    return DirDelta_WriteSig(path_iter, io.StringIO(u""), sig_outfp)
 
 
 def DirDelta(path_iter, dirsig_fileobj_list):
-    """
+    u"""
     Produce tarblock diff given dirsig_fileobj_list and pathiter
 
     dirsig_fileobj_list should either be a tar fileobj or a list of
@@ -78,20 +90,20 @@ def DirDelta(path_iter, dirsig_fileobj_list):
     """
     global stats
     stats = statistics.StatsDeltaProcess()
-    if type(dirsig_fileobj_list) is types.ListType:
-        sig_iter = combine_path_iters(map(sigtar2path_iter,
-                                          dirsig_fileobj_list))
+    if isinstance(dirsig_fileobj_list, list):
+        sig_iter = combine_path_iters([sigtar2path_iter(x) for x
+                                       in dirsig_fileobj_list])
     else:
         sig_iter = sigtar2path_iter(dirsig_fileobj_list)
     delta_iter = get_delta_iter(path_iter, sig_iter)
-    if globals.dry_run or (globals.progress and not progress.tracker.has_collected_evidence()):
+    if config.dry_run or (config.progress and not progress.tracker.has_collected_evidence()):
         return DummyBlockIter(delta_iter)
     else:
         return DeltaTarBlockIter(delta_iter)
 
 
-def delta_iter_error_handler(exc, new_path, sig_path, sig_tar = None):
-    """
+def delta_iter_error_handler(exc, new_path, sig_path, sig_tar=None):  # pylint: disable=unused-argument
+    u"""
     Called by get_delta_iter, report error in getting delta
     """
     if new_path:
@@ -99,13 +111,14 @@ def delta_iter_error_handler(exc, new_path, sig_path, sig_tar = None):
     elif sig_path:
         index_string = sig_path.get_relative_path()
     else:
-        assert 0, "Both new and sig are None for some reason"
-    log.Warn(_("Error %s getting delta for %s") % (str(exc), util.ufn(index_string)))
+        assert 0, u"Both new and sig are None for some reason"
+    log.Warn(_(u"Error %s getting delta for %s")
+             % (util.uexc(exc), util.fsdecode(index_string)))
     return None
 
 
-def get_delta_path(new_path, sig_path, sigTarFile = None):
-    """
+def get_delta_path(new_path, sig_path, sigTarFile=None):
+    u"""
     Return new delta_path which, when read, writes sig to sig_fileobj,
     if sigTarFile is not None
     """
@@ -114,35 +127,41 @@ def get_delta_path(new_path, sig_path, sigTarFile = None):
         ti = new_path.get_tarinfo()
         index = new_path.index
     delta_path = new_path.get_ropath()
-    log.Debug(_("Getting delta of %s and %s") % (new_path, sig_path))
+    log.Debug(_(u"Getting delta of %s and %s") % (new_path, sig_path))
 
     def callback(sig_string):
-        """
+        u"""
         Callback activated when FileWithSignature read to end
         """
         ti.size = len(sig_string)
-        ti.name = "signature/" + "/".join(index)
-        sigTarFile.addfile(ti, cStringIO.StringIO(sig_string))
+        if sys.version_info.major >= 3:
+            ti.name = u"signature/" + util.fsdecode(b"/".join(index))
+        else:
+            ti.name = b"signature/" + b"/".join(index)
+        sigTarFile.addfile(ti, io.BytesIO(sig_string))
 
-    if new_path.isreg() and sig_path and sig_path.isreg() and sig_path.difftype == "signature":
-        delta_path.difftype = "diff"
-        old_sigfp = sig_path.open("rb")
-        newfp = FileWithReadCounter(new_path.open("rb"))
+    if new_path.isreg() and sig_path and sig_path.isreg() and sig_path.difftype == u"signature":
+        delta_path.difftype = u"diff"
+        old_sigfp = sig_path.open(u"rb")
+        newfp = FileWithReadCounter(new_path.open(u"rb"))
         if sigTarFile:
             newfp = FileWithSignature(newfp, callback,
                                       new_path.getsize())
         delta_path.setfileobj(librsync.DeltaFile(old_sigfp, newfp))
     else:
-        delta_path.difftype = "snapshot"
+        delta_path.difftype = u"snapshot"
         if sigTarFile:
-            ti.name = "snapshot/" + "/".join(index)
+            if sys.version_info.major >= 3:
+                ti.name = u"snapshot/" + util.fsdecode(b"/".join(index))
+            else:
+                ti.name = b"snapshot/" + b"/".join(index)
         if not new_path.isreg():
             if sigTarFile:
                 sigTarFile.addfile(ti)
             if stats:
                 stats.SourceFileSize += delta_path.getsize()
         else:
-            newfp = FileWithReadCounter(new_path.open("rb"))
+            newfp = FileWithReadCounter(new_path.open(u"rb"))
             if sigTarFile:
                 newfp = FileWithSignature(newfp, callback,
                                           new_path.getsize())
@@ -152,28 +171,28 @@ def get_delta_path(new_path, sig_path, sigTarFile = None):
     return delta_path
 
 
-def log_delta_path(delta_path, new_path = None, stats = None):
-    """
+def log_delta_path(delta_path, new_path=None, stats=None):
+    u"""
     Look at delta path and log delta.  Add stats if new_path is set
     """
-    if delta_path.difftype == "snapshot":
+    if delta_path.difftype == u"snapshot":
         if new_path and stats:
             stats.add_new_file(new_path)
-        log.Info(_("A %s") %
-                 (util.ufn(delta_path.get_relative_path())),
+        log.Info(_(u"A %s") %
+                 (util.fsdecode(delta_path.get_relative_path())),
                  log.InfoCode.diff_file_new,
                  util.escape(delta_path.get_relative_path()))
     else:
         if new_path and stats:
             stats.add_changed_file(new_path)
-        log.Info(_("M %s") %
-                 (util.ufn(delta_path.get_relative_path())),
+        log.Info(_(u"M %s") %
+                 (util.fsdecode(delta_path.get_relative_path())),
                  log.InfoCode.diff_file_changed,
                  util.escape(delta_path.get_relative_path()))
 
 
 def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
-    """
+    u"""
     Generate delta iter from new Path iter and sig Path iter.
 
     For each delta path of regular file type, path.difftype with be
@@ -184,27 +203,30 @@ def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
     """
     collated = collate2iters(new_iter, sig_iter)
     if sig_fileobj:
-        sigTarFile = util.make_tarfile("w", sig_fileobj)
+        sigTarFile = util.make_tarfile(u"w", sig_fileobj)
     else:
         sigTarFile = None
     for new_path, sig_path in collated:
-        log.Debug(_("Comparing %s and %s") % (new_path and util.uindex(new_path.index),
-                                              sig_path and util.uindex(sig_path.index)))
+        log.Debug(_(u"Comparing %s and %s") % (new_path and util.uindex(new_path.index),
+                                               sig_path and util.uindex(sig_path.index)))
         if not new_path or not new_path.type:
             # File doesn't exist (but ignore attempts to delete base dir;
             # old versions of duplicity could have written out the sigtar in
             # such a way as to fool us; LP: #929067)
             if sig_path and sig_path.exists() and sig_path.index != ():
                 # but signature says it did
-                log.Info(_("D %s") %
-                         (util.ufn(sig_path.get_relative_path())),
+                log.Info(_(u"D %s") %
+                         (util.fsdecode(sig_path.get_relative_path())),
                          log.InfoCode.diff_file_deleted,
                          util.escape(sig_path.get_relative_path()))
                 if sigTarFile:
                     ti = ROPath(sig_path.index).get_tarinfo()
-                    ti.name = "deleted/" + "/".join(sig_path.index)
+                    if sys.version_info.major >= 3:
+                        ti.name = u"deleted/" + util.uindex(sig_path.index)
+                    else:
+                        ti.name = b"deleted/" + b"/".join(sig_path.index)
                     sigTarFile.addfile(ti)
-                stats.add_deleted_file()
+                stats.add_deleted_file(sig_path)
                 yield ROPath(sig_path.index)
         elif not sig_path or new_path != sig_path:
             # Must calculate new signature and create delta
@@ -226,28 +248,31 @@ def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
 
 
 def sigtar2path_iter(sigtarobj):
-    """
+    u"""
     Convert signature tar file object open for reading into path iter
     """
-    tf = util.make_tarfile("r", sigtarobj)
+    tf = util.make_tarfile(u"r", sigtarobj)
     tf.debug = 1
     for tarinfo in tf:
         tiname = util.get_tarinfo_name(tarinfo)
-        for prefix in ["signature/", "snapshot/", "deleted/"]:
+        for prefix in [r"signature/", r"snapshot/", r"deleted/"]:
             if tiname.startswith(prefix):
                 # strip prefix and '/' from name and set it to difftype
                 name, difftype = tiname[len(prefix):], prefix[:-1]
                 break
         else:
-            raise DiffDirException("Bad tarinfo name %s" % (tiname,))
+            raise DiffDirException(u"Bad tarinfo name %s" % (tiname,))
 
-        index = tuple(name.split("/"))
+        if sys.version_info.major >= 3:
+            index = tuple(util.fsencode(name).split(b"/"))
+        else:
+            index = tuple(name.split(b"/"))
         if not index[-1]:
-            index = index[:-1] # deal with trailing /, ""
+            index = index[:-1]  # deal with trailing /, ""
 
         ropath = ROPath(index)
         ropath.difftype = difftype
-        if difftype == "signature" or difftype == "snapshot":
+        if difftype == u"signature" or difftype == u"snapshot":
             ropath.init_from_tarinfo(tarinfo)
             if ropath.isreg():
                 ropath.setfileobj(tf.extractfile(tarinfo))
@@ -256,7 +281,7 @@ def sigtar2path_iter(sigtarobj):
 
 
 def collate2iters(riter1, riter2):
-    """
+    u"""
     Collate two iterators.
 
     The elements yielded by each iterator must be have an index
@@ -268,7 +293,7 @@ def collate2iters(riter1, riter2):
     while 1:
         if not relem1:
             try:
-                relem1 = riter1.next()
+                relem1 = next(riter1)
             except StopIteration:
                 if relem2:
                     yield (None, relem2)
@@ -278,7 +303,7 @@ def collate2iters(riter1, riter2):
             index1 = relem1.index
         if not relem2:
             try:
-                relem2 = riter2.next()
+                relem2 = next(riter2)
             except StopIteration:
                 if relem1:
                     yield (relem1, None)
@@ -300,7 +325,7 @@ def collate2iters(riter1, riter2):
 
 
 def combine_path_iters(path_iter_list):
-    """
+    u"""
     Produce new iterator by combining the iterators in path_iter_list
 
     This new iter will iterate every path that is in path_iter_list in
@@ -311,21 +336,21 @@ def combine_path_iters(path_iter_list):
     This is used to combine signature iters, as the output will be a
     full up-to-date signature iter.
     """
-    path_iter_list = path_iter_list[:] # copy before destructive reverse
+    path_iter_list = path_iter_list[:]  # copy before destructive reverse
     path_iter_list.reverse()
 
     def get_triple(iter_index):
-        """
+        u"""
         Represent the next element as a triple, to help sorting
         """
         try:
-            path = path_iter_list[iter_index].next()
+            path = next(path_iter_list[iter_index])
         except StopIteration:
             return None
         return (path.index, iter_index, path)
 
     def refresh_triple_list(triple_list):
-        """
+        u"""
         Update all elements with path_index same as first element
         """
         path_index = triple_list[0][0]
@@ -340,10 +365,9 @@ def combine_path_iters(path_iter_list):
                 else:
                     del triple_list[iter_index]
             else:
-                break # assumed triple_list sorted, so can exit now
+                break  # assumed triple_list sorted, so can exit now
 
-    triple_list = filter(lambda x: x, map(get_triple,
-                                          range(len(path_iter_list))))
+    triple_list = [x for x in map(get_triple, list(range(len(path_iter_list)))) if x]
     while triple_list:
         triple_list.sort()
         yield triple_list[0][2]
@@ -351,7 +375,7 @@ def combine_path_iters(path_iter_list):
 
 
 def DirDelta_WriteSig(path_iter, sig_infp_list, newsig_outfp):
-    """
+    u"""
     Like DirDelta but also write signature into sig_fileobj
 
     Like DirDelta, sig_infp_list can be a tar fileobj or a sorted list
@@ -360,38 +384,39 @@ def DirDelta_WriteSig(path_iter, sig_infp_list, newsig_outfp):
     """
     global stats
     stats = statistics.StatsDeltaProcess()
-    if type(sig_infp_list) is types.ListType:
+    if isinstance(sig_infp_list, list):
         sig_path_iter = get_combined_path_iter(sig_infp_list)
     else:
         sig_path_iter = sigtar2path_iter(sig_infp_list)
     delta_iter = get_delta_iter(path_iter, sig_path_iter, newsig_outfp)
-    if globals.dry_run or (globals.progress and not progress.tracker.has_collected_evidence()):
+    if config.dry_run or (config.progress and not progress.tracker.has_collected_evidence()):
         return DummyBlockIter(delta_iter)
     else:
         return DeltaTarBlockIter(delta_iter)
 
 
 def get_combined_path_iter(sig_infp_list):
-    """
+    u"""
     Return path iter combining signatures in list of open sig files
     """
-    return combine_path_iters(map(sigtar2path_iter, sig_infp_list))
+    return combine_path_iters([sigtar2path_iter(x) for x in sig_infp_list])
 
 
-class FileWithReadCounter:
-    """
+class FileWithReadCounter(object):
+    u"""
     File-like object which also computes amount read as it is read
     """
     def __init__(self, infile):
-        """FileWithReadCounter initializer"""
+        u"""FileWithReadCounter initializer"""
         self.infile = infile
 
-    def read(self, length = -1):
+    def read(self, length=-1):
         try:
             buf = self.infile.read(length)
-        except IOError, ex:
-            buf = ""
-            log.Warn(_("Error %s getting delta for %s") % (str(ex), util.ufn(self.infile.name)))
+        except IOError as ex:
+            buf = b""
+            log.Warn(_(u"Error %s getting delta for %s")
+                     % (util.uexc(ex), util.fsdecode(self.infile.name)))
         if stats:
             stats.SourceFileSize += len(buf)
         return buf
@@ -400,13 +425,14 @@ class FileWithReadCounter:
         return self.infile.close()
 
 
-class FileWithSignature:
-    """
+class FileWithSignature(object):
+    u"""
     File-like object which also computes signature as it is read
     """
     blocksize = 32 * 1024
+
     def __init__(self, infile, callback, filelen, *extra_args):
-        """
+        u"""
         FileTee initializer
 
         The object will act like infile, but whenever it is read it
@@ -421,7 +447,7 @@ class FileWithSignature:
         self.activated_callback = None
         self.extra_args = extra_args
 
-    def read(self, length = -1):
+    def read(self, length=-1):
         buf = self.infile.read(length)
         self.sig_gen.update(buf)
         return buf
@@ -436,20 +462,20 @@ class FileWithSignature:
         return self.infile.close()
 
 
-class TarBlock:
-    """
+class TarBlock(object):
+    u"""
     Contain information to add next file to tar
     """
     def __init__(self, index, data):
-        """
+        u"""
         TarBlock initializer - just store data
         """
         self.index = index
         self.data = data
 
 
-class TarBlockIter:
-    """
+class TarBlockIter(object):
+    u"""
     A bit like an iterator, yield tar blocks given input iterator
 
     Unlike an iterator, however, control over the maximum size of a
@@ -457,52 +483,52 @@ class TarBlockIter:
     get_footer() is available.
     """
     def __init__(self, input_iter):
-        """
+        u"""
         TarBlockIter initializer
         """
         self.input_iter = input_iter
-        self.offset = 0l                    # total length of data read
-        self.process_waiting = False        # process_continued has more blocks
-        self.process_next_vol_number = None # next volume number to write in multivol
-        self.previous_index = None          # holds index of last block returned
-        self.previous_block = None          # holds block of last block returned
-        self.remember_next = False          # see remember_next_index()
-        self.remember_value = None          # holds index of next block
-        self.remember_block = None          # holds block of next block
-        self.queued_data = None             # data to return in next next() call
+        self.offset = 0  # total length of data read
+        self.process_waiting = False  # process_continued has more blocks
+        self.process_next_vol_number = None  # next volume number to write in multivol
+        self.previous_index = None  # holds index of last block returned
+        self.previous_block = None  # holds block of last block returned
+        self.remember_next = False  # see remember_next_index()
+        self.remember_value = None  # holds index of next block
+        self.remember_block = None  # holds block of next block
+        self.queued_data = None  # data to return in next next() call
 
-    def tarinfo2tarblock(self, index, tarinfo, file_data = ""):
-        """
+    def tarinfo2tarblock(self, index, tarinfo, file_data=b""):
+        u"""
         Make tarblock out of tarinfo and file data
         """
         tarinfo.size = len(file_data)
-        headers = tarinfo.tobuf(errors='replace')
-        blocks, remainder = divmod(tarinfo.size, tarfile.BLOCKSIZE) #@UnusedVariable
+        headers = tarinfo.tobuf(errors=u'replace', encoding=config.fsencoding)
+        blocks, remainder = divmod(tarinfo.size, tarfile.BLOCKSIZE)
         if remainder > 0:
-            filler_data = "\0" * (tarfile.BLOCKSIZE - remainder)
+            filler_data = b"\0" * (tarfile.BLOCKSIZE - remainder)
         else:
-            filler_data = ""
-        return TarBlock(index, "%s%s%s" % (headers, file_data, filler_data))
+            filler_data = b""
+        return TarBlock(index, b"%s%s%s" % (headers, file_data, filler_data))
 
-    def process(self, val):
-        """
+    def process(self, val):  # pylint: disable=unused-argument
+        u"""
         Turn next value of input_iter into a TarBlock
         """
         assert not self.process_waiting
-        XXX # Override in subclass @UndefinedVariable
+        XXX  # Override in subclass @UndefinedVariable
 
     def process_continued(self):
-        """
+        u"""
         Get more tarblocks
 
         If processing val above would produce more than one TarBlock,
         get the rest of them by calling process_continue.
         """
         assert self.process_waiting
-        XXX # Override in subclass @UndefinedVariable
+        XXX  # Override in subclass @UndefinedVariable
 
-    def next(self):
-        """
+    def __next__(self):
+        u"""
         Return next block and update offset
         """
         if self.queued_data is not None:
@@ -512,10 +538,10 @@ class TarBlockIter:
             return result
 
         if self.process_waiting:
-            result = self.process_continued()
+            result = self.process_continued()  # pylint: disable=assignment-from-no-return
         else:
             # Below a StopIteration exception will just be passed upwards
-            result = self.process(self.input_iter.next())
+            result = self.process(next(self.input_iter))  # pylint: disable=assignment-from-no-return
         block_number = self.process_next_vol_number
         self.offset += len(result.data)
         self.previous_index = result.index
@@ -534,19 +560,19 @@ class TarBlockIter:
         return 64 * 1024
 
     def get_previous_index(self):
-        """
+        u"""
         Return index of last tarblock, or None if no previous index
         """
         return self.previous_index, self.previous_block
 
     def queue_index_data(self, data):
-        """
+        u"""
         Next time next() is called, we will return data instead of processing
         """
         self.queued_data = data
 
     def remember_next_index(self):
-        """
+        u"""
         When called, remember the index of the next block iterated
         """
         self.remember_next = True
@@ -554,29 +580,29 @@ class TarBlockIter:
         self.remember_block = None
 
     def recall_index(self):
-        """
+        u"""
         Retrieve index remembered with remember_next_index
         """
         return self.remember_value, self.remember_block
 
     def get_footer(self):
-        """
+        u"""
         Return closing string for tarfile, reset offset
         """
-        blocks, remainder = divmod(self.offset, tarfile.RECORDSIZE) #@UnusedVariable
-        self.offset = 0l
-        return '\0' * (tarfile.RECORDSIZE - remainder) # remainder can be 0
+        blocks, remainder = divmod(self.offset, tarfile.RECORDSIZE)
+        self.offset = 0
+        return b'\0' * (tarfile.RECORDSIZE - remainder)  # remainder can be 0
 
-    def __iter__(self):
+    def __iter__(self):  # pylint: disable=non-iterator-returned
         return self
 
 
 class DummyBlockIter(TarBlockIter):
-    """
+    u"""
     TarBlockIter that does no file reading
     """
     def process(self, delta_ropath):
-        """
+        u"""
         Get a fake tarblock from delta_ropath
         """
         ti = delta_ropath.get_tarinfo()
@@ -596,28 +622,32 @@ class DummyBlockIter(TarBlockIter):
 
 
 class SigTarBlockIter(TarBlockIter):
-    """
+    u"""
     TarBlockIter that yields blocks of a signature tar from path_iter
     """
     def process(self, path):
-        """
+        u"""
         Return associated signature TarBlock from path
         """
         ti = path.get_tarinfo()
         if path.isreg():
-            sfp = librsync.SigFile(path.open("rb"),
+            sfp = librsync.SigFile(path.open(u"rb"),
                                    get_block_size(path.getsize()))
             sigbuf = sfp.read()
             sfp.close()
-            ti.name = "signature/" + "/".join(path.index)
+            ti.name = b"signature/" + b"/".join(path.index)
+            if sys.version_info.major >= 3:
+                ti.name = util.fsdecode(ti.name)
             return self.tarinfo2tarblock(path.index, ti, sigbuf)
         else:
-            ti.name = "snapshot/" + "/".join(path.index)
+            ti.name = b"snapshot/" + b"/".join(path.index)
+            if sys.version_info.major >= 3:
+                ti.name = util.fsdecode(ti.name)
             return self.tarinfo2tarblock(path.index, ti)
 
 
 class DeltaTarBlockIter(TarBlockIter):
-    """
+    u"""
     TarBlockIter that yields parts of a deltatar file
 
     Unlike SigTarBlockIter, the argument to __init__ is a
@@ -625,15 +655,15 @@ class DeltaTarBlockIter(TarBlockIter):
     calculated.
     """
     def process(self, delta_ropath):
-        """
+        u"""
         Get a tarblock from delta_ropath
         """
         def add_prefix(tarinfo, prefix):
-            """Add prefix to the name of a tarinfo file"""
-            if tarinfo.name == ".":
-                tarinfo.name = prefix + "/"
+            u"""Add prefix to the name of a tarinfo file"""
+            if tarinfo.name == r".":
+                tarinfo.name = prefix + r"/"
             else:
-                tarinfo.name = "%s/%s" % (prefix, tarinfo.name)
+                tarinfo.name = r"%s/%s" % (prefix, tarinfo.name)
 
         ti = delta_ropath.get_tarinfo()
         index = delta_ropath.index
@@ -641,29 +671,29 @@ class DeltaTarBlockIter(TarBlockIter):
         # Return blocks of deleted files or fileless snapshots
         if not delta_ropath.type or not delta_ropath.fileobj:
             if not delta_ropath.type:
-                add_prefix(ti, "deleted")
+                add_prefix(ti, r"deleted")
             else:
-                assert delta_ropath.difftype == "snapshot"
-                add_prefix(ti, "snapshot")
+                assert delta_ropath.difftype == u"snapshot"
+                add_prefix(ti, r"snapshot")
             return self.tarinfo2tarblock(index, ti)
 
         # Now handle single volume block case
-        fp = delta_ropath.open("rb")
+        fp = delta_ropath.open(u"rb")
         data, last_block = self.get_data_block(fp)
         if stats:
             stats.RawDeltaSize += len(data)
         if last_block:
-            if delta_ropath.difftype == "snapshot":
-                add_prefix(ti, "snapshot")
-            elif delta_ropath.difftype == "diff":
-                add_prefix(ti, "diff")
+            if delta_ropath.difftype == u"snapshot":
+                add_prefix(ti, r"snapshot")
+            elif delta_ropath.difftype == u"diff":
+                add_prefix(ti, r"diff")
             else:
-                assert 0, "Unknown difftype"
+                assert 0, u"Unknown difftype"
             return self.tarinfo2tarblock(index, ti, data)
 
         # Finally, do multivol snapshot or diff case
-        full_name = "multivol_%s/%s" % (delta_ropath.difftype, ti.name)
-        ti.name = full_name + "/1"
+        full_name = r"multivol_%s/%s" % (delta_ropath.difftype, ti.name)
+        ti.name = full_name + r"/1"
         self.process_prefix = full_name
         self.process_fp = fp
         self.process_ropath = delta_ropath
@@ -672,26 +702,26 @@ class DeltaTarBlockIter(TarBlockIter):
         return self.tarinfo2tarblock(index, ti, data)
 
     def get_data_block(self, fp):
-        """
+        u"""
         Return pair (next data block, boolean last data block)
         """
         read_size = self.get_read_size()
         buf = fp.read(read_size)
         if len(buf) < read_size:
             if fp.close():
-                raise DiffDirException("Error closing file")
+                raise DiffDirException(u"Error closing file")
             return (buf, True)
         else:
             return (buf, False)
 
     def process_continued(self):
-        """
+        u"""
         Return next volume in multivol diff or snapshot
         """
         assert self.process_waiting
         ropath = self.process_ropath
         ti, index = ropath.get_tarinfo(), ropath.index
-        ti.name = "%s/%d" % (self.process_prefix, self.process_next_vol_number)
+        ti.name = u"%s/%d" % (self.process_prefix, self.process_next_vol_number)
         data, last_block = self.get_data_block(self.process_fp)
         if stats:
             stats.RawDeltaSize += len(data)
@@ -707,13 +737,13 @@ class DeltaTarBlockIter(TarBlockIter):
 
 
 def write_block_iter(block_iter, out_obj):
-    """
+    u"""
     Write block_iter to filename, path, or file object
     """
     if isinstance(out_obj, Path):
-        fp = open(out_obj.name, "wb")
-    elif type(out_obj) is types.StringType:
-        fp = open(out_obj, "wb")
+        fp = open(out_obj.name, u"wb")
+    elif isinstance(out_obj, (str, u"".__class__)):
+        fp = open(out_obj, u"wb")
     else:
         fp = out_obj
     for block in block_iter:
@@ -725,7 +755,7 @@ def write_block_iter(block_iter, out_obj):
 
 
 def get_block_size(file_len):
-    """
+    u"""
     Return a reasonable block size to use on files of length file_len
 
     If the block size is too big, deltas will be bigger than is
@@ -733,8 +763,8 @@ def get_block_size(file_len):
     patching can take a really long time.
     """
     if file_len < 1024000:
-        return 512 # set minimum of 512 bytes
+        return 512  # set minimum of 512 bytes
     else:
         # Split file into about 2000 pieces, rounding to 512
-        file_blocksize = long((file_len / (2000 * 512)) * 512)
-        return min(file_blocksize, globals.max_blocksize)
+        file_blocksize = int((file_len / (2000 * 512))) * 512
+        return min(file_blocksize, config.max_blocksize)
