@@ -366,7 +366,10 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
         mf = config.restart.last_backup.get_local_manifest()
         config.restart.checkManifest(mf)
         config.restart.setLastSaved(mf)
-        validate_encryption_settings(config.restart.last_backup, mf)
+        if not (config.s3_use_deep_archive or config.s3_use_glacier):
+            validate_encryption_settings(config.restart.last_backup, mf)
+        else:
+            log.Warn(_(u"Skipping encryption validation due to glacier/deep storage"))
         mf.fh = man_outfp
         last_block = config.restart.last_block
         log.Notice(_(u"Restarting after volume %s, file %s, block %s") %
@@ -745,6 +748,10 @@ def restore_get_patched_rop_iter(col_stats):
         u"""Get file object iterator from backup_set contain given index"""
         manifest = backup_set.get_manifest()
         volumes = manifest.get_containing_volumes(index)
+
+        if hasattr(backup_set.backend.backend, u'pre_process_download_batch'):
+            backup_set.backend.backend.pre_process_download_batch(backup_set.volume_name_dict.values())
+
         for vol_num in volumes:
             yield restore_get_enc_fileobj(backup_set.backend,
                                           backup_set.volume_name_dict[vol_num],
@@ -1602,8 +1609,6 @@ def do_backup(action):
         action = u"full"
     log.PrintCollectionStatus(col_stats)
 
-    os.umask(0o77)
-
     # get the passphrase if we need to based on action/options
     config.gpg_profile.passphrase = get_passphrase(1, action)
 
