@@ -221,7 +221,8 @@ class ROPath(object):
         self.mode = tarinfo.mode
         self.stat = StatResult()
 
-        u""" Set user and group id
+        u""" If do_not_restore_owner is False,
+        set user and group id
         use numeric id if name lookup fails
         OR
         --numeric-owner is set
@@ -405,7 +406,7 @@ class ROPath(object):
             else:
                 return 1
         elif self.issym():
-            if self.symtext == other.symtext or self.symtext + u"/" == other.symtext:
+            if self.symtext == other.symtext or self.symtext + util.fsencode(os.sep) == other.symtext:
                 return 1
             else:
                 log_diff(_(u"Symlink %%s points to %s, expected %s") %
@@ -456,7 +457,8 @@ class ROPath(object):
             os.mkdir(other.name)
         elif self.issym():
             os.symlink(self.symtext, other.name)
-            os.lchown(other.name, self.stat.st_uid, self.stat.st_gid)
+            if not config.do_not_restore_ownership:
+                os.lchown(other.name, self.stat.st_uid, self.stat.st_gid)
             other.setdata()
             return  # no need to copy symlink attributes
         elif self.isfifo():
@@ -474,7 +476,7 @@ class ROPath(object):
     def copy_attribs(self, other):
         u"""Only copy attributes from self to other"""
         if isinstance(other, Path):
-            if self.stat is not None:
+            if self.stat and not config.do_not_restore_ownership:
                 util.maybe_ignore_errors(lambda: os.chown(other.name, self.stat.st_uid, self.stat.st_gid))
             util.maybe_ignore_errors(lambda: os.chmod(other.name, self.mode))
             util.maybe_ignore_errors(lambda: os.utime(other.name, (time.time(), self.stat.st_mtime)))
@@ -511,7 +513,7 @@ class Path(ROPath):
             path, extra = os.path.split(path)
             tail.insert(0, extra)
         if path:
-            return config.rename[path].split(os.sep) + tail
+            return config.rename[path].split(util.fsencode(os.sep)) + tail
         else:
             return index  # no rename found
 
@@ -576,6 +578,14 @@ class Path(ROPath):
         u"""Return true if path is a directory and is empty"""
         return self.isdir() and not self.listdir()
 
+    def contains(self, child):
+        u"""Return true if path is a directory and contains child"""
+        if isinstance(child, u"".__class__):
+            child = util.fsencode(child)
+        # We don't use append(child).exists() here because that requires exec
+        # permissions as well as read. listdir() just needs read permissions.
+        return self.isdir() and child in self.listdir()
+
     def open(self, mode=u"rb"):
         u"""
         Return fileobj associated with self
